# Generated by Django 3.1.7 on 2021-03-22 16:57

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ryzom_django_channels', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='publication',
            name='template_class',
        ),
        migrations.RemoveField(
            model_name='publication',
            name='template_module',
        ),
    ]
